"use strict";
var __importDefault = (this && this.__importDefault) || function (mod) {
    return (mod && mod.__esModule) ? mod : { "default": mod };
};
Object.defineProperty(exports, "__esModule", { value: true });
exports.getClassPuml = void 0;
const endent_1 = __importDefault(require("endent"));
const modifiers_1 = require("../parser/modifiers");
const modifier_1 = require("./modifier");
/**
 * Converts a Java Class to PlantUML code.
 * @param javaClass the Java Class(es) to convert
 * @param prefix [for inner classes]
 * @returns the generated PlantUML code (without @startuml/@enduml)
 */
function getClassPuml(javaClass) {
    if (!('name' in javaClass)) {
        return javaClass.map((jc) => getClassPuml(jc)).join('\n\n');
    }
    // if (javaClass.name.includes('Leven')) console.log('Leven Prefix:', prefix);
    // if (javaClass.classes.length !== 0) console.log(javaClass, getClassName(javaClass.name.split('.').at(-1) ?? 'OuterClass', prefix));
    return (0, endent_1.default) `
        ${getClassType(javaClass.modifiers)} ${javaClass.name} {
            ${javaClass.values ? `' --- values (${javaClass.name}) ---` : ''}
            ${javaClass.values?.join(', ') ?? ''}

            ${javaClass.fields.length ? `' --- fields (${javaClass.name}) ---` : ''}
            ${getFieldPuml(javaClass.fields)}

            ${javaClass.methods.length ? `' --- methods (${javaClass.name}) ---` : ''}
            ${getMethodPuml(javaClass.methods)}
        }
        ${getClassPuml(javaClass.classes)}
        ${getInnerClassConnections(javaClass.name, javaClass.classes).join('\n')}`;
}
exports.getClassPuml = getClassPuml;
/**
 * converts Modifiers to PlantUML code.
 * @param javaModifiers the classes Modifiers
 * @returns the PlantUML equivalent
 */
function getClassType(javaModifiers) {
    if (javaModifiers.includes(modifiers_1.Modifier.class))
        return 'class';
    if (javaModifiers.includes(modifiers_1.Modifier.interface))
        return 'interface';
    if (javaModifiers.includes(modifiers_1.Modifier.enum))
        return 'enum';
    if (javaModifiers.includes(modifiers_1.Modifier.abstract))
        return 'abstract';
    return 'class';
}
/**
 * Converts a Java Field to PlantUML code.
 * @param javaField the Java Field to convert
 * @returns the resulting plantuml string
 */
function getFieldPuml(javaField) {
    if (!('name' in javaField)) {
        return javaField.map((jf) => getFieldPuml(jf)).join('\n');
    }
    return `${(0, modifier_1.getModifierPuml)(javaField.modifiers)}${javaField.type} ${javaField.name}`;
}
/**
 * Converts a Java Method to PlantUML code.
 * @param javaMethod the Java Method to convert
 * @returns the resulting plantuml string
 */
function getMethodPuml(javaMethod) {
    if (!('name' in javaMethod)) {
        return javaMethod.map((jm) => getMethodPuml(jm)).join('\n');
    }
    return (`${(0, modifier_1.getModifierPuml)(javaMethod.modifiers)}${javaMethod.returnType} ` +
        `${javaMethod.name}(${javaMethod.parameters.map(({ name, type }) => `${type} ${name}`).join(', ')})`);
}
function getInnerClassConnections(name, classes) {
    return classes.map((inner) => `${name} +-- ${inner.name}`);
}
