"use strict";
var __createBinding = (this && this.__createBinding) || (Object.create ? (function(o, m, k, k2) {
    if (k2 === undefined) k2 = k;
    Object.defineProperty(o, k2, { enumerable: true, get: function() { return m[k]; } });
}) : (function(o, m, k, k2) {
    if (k2 === undefined) k2 = k;
    o[k2] = m[k];
}));
var __setModuleDefault = (this && this.__setModuleDefault) || (Object.create ? (function(o, v) {
    Object.defineProperty(o, "default", { enumerable: true, value: v });
}) : function(o, v) {
    o["default"] = v;
});
var __importStar = (this && this.__importStar) || function (mod) {
    if (mod && mod.__esModule) return mod;
    var result = {};
    if (mod != null) for (var k in mod) if (k !== "default" && Object.prototype.hasOwnProperty.call(mod, k)) __createBinding(result, mod, k);
    __setModuleDefault(result, mod);
    return result;
};
Object.defineProperty(exports, "__esModule", { value: true });
exports.findJavaFiles = exports.findSubPackages = exports.findPackage = void 0;
const glob_promise_1 = require("glob-promise");
const node_fs_1 = require("node:fs");
const node_path_1 = __importStar(require("node:path"));
/**
 * Finds the directory of a package in the given root directory.
 * @param root the root directory
 * @param name the package name that should be found
 * @returns the directory of the package
 */
async function findPackage(root, name) {
    root = (0, node_path_1.normalize)(root);
    if (name) {
        const packageDir = node_path_1.join.apply(null, name.split('.'));
        const needs = (0, node_path_1.join)('src', 'main', 'java', packageDir);
        if (root.includes(needs))
            return trimPath(root, needs);
        const wanted = (0, node_path_1.join)(root, needs);
        if (!(0, node_fs_1.existsSync)(wanted))
            return;
        return (0, node_fs_1.statSync)(wanted).isDirectory() ? wanted : undefined;
    }
    // no src/main/java with name in root found
    if (root.includes((0, node_path_1.join)('src', 'main', 'java'))) {
        root = await trimPath(root, (0, node_path_1.join)('src', 'main', 'java'));
    }
    else {
        if (!(0, node_fs_1.existsSync)((0, node_path_1.join)(root, 'src', 'main', 'java')))
            return;
        if (!(0, node_fs_1.statSync)((0, node_path_1.join)(root, 'src', 'main', 'java')).isDirectory())
            return;
        root = (0, node_path_1.join)(root, 'src', 'main', 'java');
    }
    // root now ends with src/main/java
    if (name) {
        const wanted = (0, node_path_1.join)(root, node_path_1.join.apply(null, name.split('.')));
        if (!(0, node_fs_1.existsSync)(wanted))
            return;
        return (0, node_fs_1.statSync)(wanted).isDirectory() ? wanted : undefined;
    }
    // return the 'biggest' package in src/main/java
    const globbed = await (0, glob_promise_1.promise)((0, node_path_1.join)(root, '**', '*.java'));
    if (globbed.length === 0)
        return;
    // find directory that contains every found file
    return findCommonDirectory(globbed);
}
exports.findPackage = findPackage;
async function findSubPackages(root) {
    return (await (0, glob_promise_1.promise)((0, node_path_1.join)(root, '*')))
        .filter((value) => (0, node_fs_1.existsSync)(value) && (0, node_fs_1.statSync)(value).isDirectory())
        .filter(async (value) => containsJavaFiles(value))
        .map((value) => value.replaceAll('/', node_path_1.default.sep))
        .map((value) => {
        return { root: value, name: value.split(node_path_1.default.sep).at(-1) };
    });
}
exports.findSubPackages = findSubPackages;
async function containsJavaFiles(root) {
    return (await (0, glob_promise_1.promise)((0, node_path_1.join)(root, '**', '*.java'))).length !== 0;
}
async function findJavaFiles(root) {
    return (await (0, glob_promise_1.promise)((0, node_path_1.join)(root, '*.java')))
        .filter((value) => (0, node_fs_1.existsSync)(value) && (0, node_fs_1.statSync)(value).isFile)
        .map((value) => value.replaceAll('/', node_path_1.default.sep));
}
exports.findJavaFiles = findJavaFiles;
async function trimPath(root, needs) {
    let current = root;
    let next = (0, node_path_1.parse)(root).dir;
    while (next.includes(needs)) {
        current = next;
        next = (0, node_path_1.parse)(next).dir;
    }
    return current;
}
function findCommonDirectory(dirs) {
    // create 2D array of all directory names
    const dirArray = dirs.map((value) => (0, node_path_1.normalize)(value).split(node_path_1.default.sep)).sort((a, b) => a.length - b.length);
    const resultArray = [];
    while (dirArray.at(0)?.length ?? 0 > 0) {
        const toBeAdded = dirArray[0][0];
        const sameness = dirArray.map((value) => value[0] === toBeAdded).reduce((_, current) => current);
        if (!sameness)
            break;
        resultArray.push(toBeAdded);
        dirArray.forEach((innerArray) => innerArray.shift());
    }
    return resultArray.join(node_path_1.default.sep);
}
