"use strict";
var __createBinding = (this && this.__createBinding) || (Object.create ? (function(o, m, k, k2) {
    if (k2 === undefined) k2 = k;
    Object.defineProperty(o, k2, { enumerable: true, get: function() { return m[k]; } });
}) : (function(o, m, k, k2) {
    if (k2 === undefined) k2 = k;
    o[k2] = m[k];
}));
var __setModuleDefault = (this && this.__setModuleDefault) || (Object.create ? (function(o, v) {
    Object.defineProperty(o, "default", { enumerable: true, value: v });
}) : function(o, v) {
    o["default"] = v;
});
var __importStar = (this && this.__importStar) || function (mod) {
    if (mod && mod.__esModule) return mod;
    var result = {};
    if (mod != null) for (var k in mod) if (k !== "default" && Object.prototype.hasOwnProperty.call(mod, k)) __createBinding(result, mod, k);
    __setModuleDefault(result, mod);
    return result;
};
Object.defineProperty(exports, "__esModule", { value: true });
exports.getPackage = void 0;
const class_1 = require("./class");
const node_path_1 = __importStar(require("node:path"));
const files_1 = require("../files");
const promises_1 = require("node:fs/promises");
/**
 * Parses a directory and the contained java files.
 * @param root the root directory of the package
 * @param name [the name of the package]
 * @returns the package with all of its contents
 */
async function getPackage(root, name) {
    root = (0, node_path_1.normalize)(root);
    if (!name)
        name = extractName(root);
    const subPackages = await (0, files_1.findSubPackages)(root);
    const packages = [];
    let subIndex = 0;
    for (const subPackage of subPackages) {
        const subName = subPackage.name ? `${name}.${subPackage.name}` : `${name}.sub${subIndex++}`;
        packages.push(await getPackage(subPackage.root, subName));
    }
    const classes = [];
    const javaFiles = await (0, files_1.findJavaFiles)(root);
    for (const javaFile of javaFiles) {
        // console.log(getClass(await readFile(javaFile, 'utf-8')));
        classes.push((0, class_1.getClass)(await (0, promises_1.readFile)(javaFile, 'utf-8')));
    }
    // console.log(classes);
    const result = { name, packages, classes };
    matchPackageImports(result);
    return result;
}
exports.getPackage = getPackage;
/**
 * Extracts the package name from a path.
 * @param root the path
 * @returns the extracted name, 'unknown.package' if no 'src/main/java' directory was found
 */
function extractName(root) {
    let name = root.split((0, node_path_1.join)('src', 'main', 'java')).at(-1)?.replaceAll(node_path_1.default.sep, '.');
    name = name ?? 'unknown.package';
    return name.startsWith('.') ? name.replace('.', '') : name;
}
function matchPackageImports(result) {
    result.classes = result.classes.map((inheritor) => {
        for (const candidate of result.classes) {
            if (candidate.name.endsWith(`.${inheritor.extends}`)) {
                inheritor.extends = candidate.name;
            }
            inheritor.implements.map((className) => {
                if (candidate.name.endsWith(`.${className}`)) {
                    return candidate;
                }
                return className;
            });
        }
        return inheritor;
    });
    return result;
}
